@@ -1,33 +1,42 @@
-# Copyright 2007, 2008, 2010 Mario Pineda-Krch.
-#
-# This file is part of the R package GillespieSSA.
-#
-# GillespieSSA is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-
-# GillespieSSA is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with GillespieSSA.  If not, see <http://www.gnu.org/licenses/>.
-
-ssa.terminate <- function(args,out.rxn,tf,method,maxWallTime,verbose) {
+#' Terminates a simulation that was invoked using ssa
+#'
+#' Terminates a simulation that was invoked using ssa.
+#'
+#' Terminates an invocation of the \code{link{ssa}} wrapper function. Returns
+#' the same list object as [ssa()]. This function is called from
+#' within [ssa()] and is not intended to be invoked stand alone.
+#'
+#' @param args list of arguments and their values passed to [ssa()].
+#' @param out.rxn list object as returned from [ssa.run()].
+#' @param tf final time.
+#' @param method ssa method to use.
+#' @param maxWallTime maximum simulation wall time duration (in seconds) that
+#' the simulation is allowed to run for before terminated.
+#' @param verbose boolean value indicating if some basic simulation statistics
+#' should be displayed on the console.
+#' @seealso [ssa()]
+#' @keywords misc datagen ts
+#' @importFrom stats sd
+ssa.terminate <- function(
+  args,
+  out.rxn,
+  tf,
+  method,
+  maxWallTime,
+  verbose
+) {
 
   # Get the final time and state vector
   t <- out.rxn$timeSeries[dim(out.rxn$timeSeries)[1],1]
   x <- out.rxn$timeSeries[dim(out.rxn$timeSeries)[1],2:dim(out.rxn$timeSeries)[2]]
 
   # Figure out all the reasons why the simulation terminated
-  terminationStatus <- NULL 
+  terminationStatus <- NULL
   if (t>=tf)          terminationStatus <- c(terminationStatus, "finalTime")
   if (all(x==0))      terminationStatus <- c(terminationStatus, "extinction")
   if (any(x<0))       terminationStatus <- c(terminationStatus, "negativeState")
   if (all(out.rxn$eval_a==0)) terminationStatus <- c(terminationStatus, "zeroProp")
-  if (out.rxn$elapsedWallTime>=maxWallTime) terminationStatus <- c(terminationStatus, "maxWallTime") 
+  if (out.rxn$elapsedWallTime>=maxWallTime) terminationStatus <- c(terminationStatus, "maxWallTime")
 
   # Calculate some stats for the used method
   stats <- list(startWallime       = out.rxn$startWallTime,
@@ -53,10 +62,10 @@ ssa.terminate <- function(args,out.rxn,tf,method,maxWallTime,verbose) {
     }
     cat("End wall time: ",stats$endWallTime,"\n",sep="")
     cat("--------------------\n")
-    }
+  }
 
-  # Return simulation results ('chopping' off any rows in the timeSeries matrix that have no values (NA))    
-  return(list(data  = out.rxn$timeSeries, 
-              stats = stats, 
+  # Return simulation results ('chopping' off any rows in the timeSeries matrix that have no values (NA))
+  return(list(data  = out.rxn$timeSeries,
+              stats = stats,
               args  = args))
-}
\ No newline at end of file
+}
